import logging
from dataclasses import dataclass

from core.domain.worker.Worker import Worker
from core.domain.worker.WorkerLogLevel import WorkerLogLevel
from core.domain.worker.WorkerType import WorkerType
from core.repos.WorkerLogRepo import WorkerLogRepo
from core.repos.WorkerRepo import WorkerRepo
from core.repos.WorkerStatusRepo import WorkerStatusRepo
from core.services.SystemService import SystemService

log = logging.getLogger(__name__)


@dataclass
class Register_worker:
	workerRepo: WorkerRepo
	workerStatusRepo: WorkerStatusRepo
	workerLogRepo: WorkerLogRepo
	systemService: SystemService

	class Result:
		@dataclass
		class WorkerAlreadyExists:
			worker: Worker

		@dataclass
		class Ok:
			worker: Worker

	def now(self, workerType: WorkerType) -> Result.Ok | Result.WorkerAlreadyExists:
		log.info("Get current public ip")
		ip = self.systemService.getIp()

		log.info("Check if worker already exists")
		worker = self.workerRepo.getByIp(ip=ip, type=workerType)
		if worker is not None:
			return self.Result.WorkerAlreadyExists(worker=worker)

		log.info("Register worker by ip")
		worker = self.workerRepo.post(ip=ip, type=workerType)

		log.info("Log worker registration and initial status")
		self.workerLogRepo.post(context=__name__, workerId=worker.id, data=f"Registering '{workerType}' worker with ip: '{ip}'", level=WorkerLogLevel.INFO)

		return self.Result.Ok(worker=worker)
